#!/usr/bin/env ruby
######################################################################
# COPYRIGHT AND PERMISSION NOTICE
# 
# Copyright (c) 2001 Colin Steele
# 
# All rights reserved.
# 
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, provided
# that the above copyright notice(s) and this permission notice appear
# in all copies of the Software and that both the above copyright
# notice(s) and this permission notice appear in supporting
# documentation.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR HOLDERS INCLUDED IN THIS NOTICE BE LIABLE FOR
# ANY CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY
# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
# OF THIS SOFTWARE.
# 
# Except as contained in this notice, the name of a copyright holder
# shall not be used in advertising or otherwise to promote the sale,
# use or other dealings in this Software without prior written
# authorization of the copyright holder.
######################################################################

require "ruby_libgtop"

cpu = Glibtop::CPU.new
printf("CPU: %d seconds total, %0.2f%% user, " +
       "%0.2f%% nice, %0.2f%% sys, %0.2f%% idle\n",
       cpu.total,
       100.0 * cpu.user / cpu.total,
       100.0 * cpu.nice / cpu.total,
       100.0 * cpu.sys / cpu.total,
       100.0 * cpu.idle / cpu.total)

uptime = Glibtop::Uptime.new
printf("Uptime: %.0f days uptime, %.0f days idle\n",
       uptime.up / 86400,
       uptime.idle / 86400)

loadavg = Glibtop::Loadavg.new.avg
printf("Load Average: %2.2f, %2.2f, %2.2f\n",
       loadavg[0],
       loadavg[1],
       loadavg[2])

mem = Glibtop::Memory.new
puts "Memory: #{mem.total / 1024}k total - #{mem.used / 1024}k used = " +
  "#{mem.free / 1024}k free"
puts "Memory: #{mem.shared / 1024}k shared, #{mem.buffer / 1024}k buff, " +
  "#{mem.cached / 1024}k cached, #{mem.user / 1024}k user, " +
  "#{(mem.locked || 0) / 1024}k locked"

swap = Glibtop::Swap.new
puts "Swap: #{swap.total / 1024}k total - #{swap.used / 1024}k used = " +
  "#{swap.free / 1024}k free"
puts "Paging: In #{swap.page_in}, Out #{swap.page_out}"

list = Glibtop::Proclist.new.pids

sleeping = 0
zombie = 0
stopped = 0
running = 0
output = " PPID   PID  STAT   SIZE  VSIZE    RSS  SHARE NAME/ARGS\n"
list.each { |pid|
  process = Glibtop::Process.new(pid)
  sleeping +=1 if process.state == "S"
  stopped +=1 if process.state == "T"
  zombie +=1 if process.state == "Z"
  running +=1 if process.state == "R"
  args = process.args
  args.shift
  line = sprintf("%5d %5d %5s %5dk %5dk %5dk %5dk %s ",
                 process.ppid,
                 process.pid,
                 process.state,
                 (process.size || 0) / 1024,
                 (process.vsize || 0) / 1024,
                 (process.rss || 0) / 1024,
                 (process.share || 0) / 1024,
                 process.cmd)
  leftOver = 79 - line.length
  line += (args.join(" ")[0..leftOver] || "") + "\n"
  output += line
}
puts "#{list.size} processes: #{sleeping} sleeping, #{running} running, " +
  "#{stopped} stopped, #{zombie} zombie"
puts output
